#!/bin/bash
#
# Copyright (C) 2004-2011 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of le-base
#
# le-base is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

THIS=$(basename $0)
DFGROUP="freeze"
PDAPATH=".kde/share/config/plasma-desktop-appletsrc"
SKELPDA="/etc/skel/$PDAPATH"

# Print script usage
function usage() {
    echo "Uso: $THIS [opções]

Opções:
    -h                  uso
    -a USUARIO          adicionar usuário ao deepfreeze
    -r USUARIO          remover usuário do deepfreeze
    -m                  mais informações
    "
    exit -1
}

# Print more information about script usage
function info() {
    echo "Mais informações:

    É possível adicionar e remover mais de um usuário ao mesmo tempo.
    Basta passar os parâmetros -a ou -r para cada usuário, respectivamente.

    Exemplo:
    $THIS -r USUARIO -a USUARIO2
    Remove USUARIO do deepfreeze e adiciona USUARIO2 ao deepfreeze.
    "
    exit 0
}

# Validate an user
function validuser() {
    if getent passwd | grep -q "^$1:"; then
        return 0
    else
        echo "Usuário $1 não existe no sistema"
        return 1
    fi
}

# Remove an user from deep-freeze group and modify some files
function dfremove() {
    if ! validuser $1; then
        return 1
    fi

    FILE="/home/$1/$PDAPATH"
    PLUGINSTR="\[Containments\]\[1\]\[ActionPlugins\]$"
    PLUGINRBSTR="\[Containments\]\[1\]\[ActionPlugins\]\[RightButton;NoModifier\]$"
    APPSTR="^plugin=ihatethecashew$"

    # Exit if user does not belong to deep-freeze group
    if groups $1 | grep -qv "\<$DFGROUP\>"; then
        echo "Usuário $1 não pertence ao grupo $DFGROUP"
        return 1
    fi

    if test -e $FILE; then
        # User $FILE is equal to skel plasma-desktop-appletsrc (never login)
        if diff $FILE $SKELPDA > /dev/null; then
            sed -i '16,38d' $FILE
            if test $? -ne 0; then
                echo "Erro ao modificar $FILE"
                return 1
            else
                chown --reference="/home/$1" $FILE
            fi
        else
            PSL=$(grep -n $PLUGINSTR $FILE | cut -d':' -f1)
            PEL=$((PSL + 3))
            PRBSL=$(grep -n $PLUGINRBSTR $FILE | cut -d':' -f1)
            PRBEL=$((PRBSL + 18))
            ASL=$(grep -n $APPSTR $FILE | cut -d':' -f1)

            if test -n "$PSL" -a -n "$PRBSL" -a -n "$ASL"; then
                # Modify $PDA file to avoid problems on login
                bash le-fix-wallpaper.sh -q -u $1 && \
                sed -i "$PRBSL,$PRBEL d" $FILE && \
                sed -i "$PSL,$PEL d" $FILE
                if test $? -ne 0; then
                    echo "Erro ao modificar $FILE"
                    return 1
                else
                    chown --reference="/home/$1" $FILE
                fi
            elif test -n "$PSL" -o -n "$PRBSL" -o -n "$ASL"; then
                echo "$FILE está fora do padrão do sistema."
                echo "Logue com o usuário e tente removê-lo novamente"
                return 1
            fi
        fi
    fi

    # Remove user from deep-freeze group
    gpasswd -d $1 $DFGROUP > /dev/null
    if test $? -ne 0; then
        echo "Falha ao remover usuário do grupo $DFGROUP"
        return 1
    else
        echo "Usuário $1 removido do grupo $DFGROUP com sucesso"
    fi
}

# Add an user to deep-freeze group
function dfadd() {
    if ! validuser $1; then
        return 1
    fi

    if groups $1 | grep -q "\<$DFGROUP\>"; then
        echo "Usuário $1 já pertence ao grupo $DFGROUP"
        return 1
    fi

    # Add user to deep-freeze group
    gpasswd -a $1 $DFGROUP > /dev/null
    if test $? -eq 0; then
        echo "Usuário $1 adicionado ao grupo $DFGROUP com sucesso"
    else
        echo "Falha ao adicionar usuário $1 ao grupo $DFGROUP"
        return 1
    fi
}

# Verify is deep-freeze group exists
if ! $(getent group | grep -q "^${DFGROUP}:"); then
    echo "Grupo $DFGROUP inexistente, reinstale o le-base"
    exit -1
fi

while getopts "hr:a:m" OPT; do
    case $OPT in
        "h") usage;;
        "m") info;;
        "r") dfremove $OPTARG;;
        "a") dfadd $OPTARG;;
        "?") usage; exit -2;;
        *) usage;;
    esac
done

# Print usage if none parameter was passed
if test $# -eq 0; then
    usage
fi

exit 0
